import pygame
import math

import main
from gfx import loadImage
import outro

import pieces

tiles = [[pieces.Horizontal, pieces.Corner1, pieces.ValveHorizontal],
         [pieces.ThinHorizontal, pieces.TJunction1, pieces.ReleaseHorizontal],
         [pieces.ReservoirHorizontal, pieces.CompressorRight, pieces.LimitRight]]


xs = [30, 67, 104]
ys = [147, 185, 222]

class InfoBox:
    def __init__(self, game):
        self.game = game

        self.img = loadImage("infobox.png")
        self.rect = pygame.Rect(28, 28, 172, 544)
        self.font = pygame.font.Font(pygame.font.match_font("Arial"), 60)
        self.font2 = pygame.font.Font(pygame.font.match_font("Arial"), 30)

        # draw clock box
        pygame.draw.rect(self.img, (0, 0, 0), pygame.Rect(5, 5, 162, 64), 1)

        # draw score box
        pygame.draw.rect(self.img, (0, 0, 0), pygame.Rect(5, 74, 162, 32), 1)

        # draw lives box
        pygame.draw.rect(self.img, (0, 0, 0), pygame.Rect(5, 111, 162, 32), 1)
        self.heart = loadImage("heart.png")

        # draw quit button
        pygame.draw.rect(self.img, (0, 0, 0), pygame.Rect(5, 475, 162, 64), 1)
        quit = self.font.render("Menu", True, (0, 0, 0))
        self.quitrect = quit.get_rect()
        self.quitrect.topleft = (86-(self.quitrect.width/2), 507-(self.quitrect.height/2))
        self.img.blit(quit, self.quitrect)
        self.quitrect = pygame.Rect(5+28, 475+28, 162, 64)

        # draw tile boxes
        for y, row in zip(ys, tiles):
            for x, tile in zip(xs, row):
                pygame.draw.rect(self.img, (0, 0, 0), pygame.Rect(x, y, 32, 32), 1)
                if tile is not None:
                    t = tile()
                    t.rect.topleft = (x, y)
                    self.img.blit(t.image, t.rect)
                    if isinstance(t, pieces.ValveHorizontal):
                        self.img.blit(t.handle, t.rect)
        self.tilerect = pygame.Rect(30+28, 147+28, 111, 111)

        # draw other options
        pygame.draw.rect(self.img, (0, 0, 0), pygame.Rect(30, 270, 32, 32), 1)
        pygame.draw.rect(self.img, (0, 0, 0), pygame.Rect(67, 270, 32, 32), 1)
        pygame.draw.rect(self.img, (0, 0, 0), pygame.Rect(104, 270, 32, 32), 1)

        self.pause = loadImage("pause.png")
        self.play = loadImage("play.png")
        self.playrect = self.play.get_rect()
        self.playrect.topleft = (30+28, 270+28)
        spanner = loadImage("spanner.png")
        self.img.blit(spanner, pygame.Rect(67, 270, 32, 32))
        self.spannerrect = pygame.Rect(28+67, 28+270, 32, 32)
        bomb = loadImage("bomb.png")
        self.img.blit(bomb, pygame.Rect(104, 270, 32, 32))
        self.bombrect = pygame.Rect(28+104, 28+270, 32, 32)

        # draw the graphs
        pygame.draw.rect(self.img, (0, 0, 0), pygame.Rect(5, 337, 12, 128), 1)
        pygame.draw.rect(self.img, (0, 0, 0), pygame.Rect(155, 337, 12, 128), 1)

        # draw the main wheel!
        pygame.draw.rect(self.img, (0, 0, 0), pygame.Rect(22, 337, 128, 128), 1)
        self.wheel_normal = loadImage("wheel.png")
        self.wheel = pygame.transform.rotate(self.wheel_normal, 0.0)
        self.wheelrect = pygame.Rect(22+28, 337+28, 128, 128)

        self.time_left = 60*5

    def update(self, time):
        if main.state == main.PLAY:
            self.time_left = self.time_left - time
            if self.time_left <= 0:
                main.gm.lives -= 1
                main.outro = outro.Outro(False)
                main.state = main.LEVEL_OUTRO

    def render(self, screen):
        minutes = self.time_left / 60
        seconds = (self.time_left) % 60
        clock = self.font.render("%i:%.2i" % (minutes, seconds), True, (0, 0, 0))
        clockrect = clock.get_rect()
        clockrect.topleft = (28+86-(clockrect.width/2), 28+37-(clockrect.height/2))

        score = self.font2.render("%i" % (main.gm.score), True, (0, 0, 0))
        scorerect = score.get_rect()
        scorerect.topleft = (28+86-(scorerect.width/2), 28+90-(scorerect.height/2))

        screen.blit(self.img, self.rect)

        heartrect = pygame.Rect(0, 111+28, 32, 32)
        x = 28 + 5 + 81 - main.gm.lives*16
        for i in range(main.gm.lives):
            heartrect.topleft = (x, 111+28)
            screen.blit(self.heart, heartrect)
            x += 32

        # draw graphs
        total, target = main.gm.machine.total, main.gm.machine.target
        x = 28+6
        y = 28+337+128
        overall_total = float(target["O"] + target["R"] + target["G"] + target["B"] + target["Y"])
        for k, c in [("O", (150, 150, 150)), ("R", (255, 0, 0)), ("G", (0, 255, 0)), ("B", (0, 0, 255)), ("Y", (255, 255, 0))]:
            if target[k] == 0:
                continue
            amount = int(128*(float(total[k])/target[k])*(target[k]/overall_total))
            screen.fill(c, pygame.Rect(x, y-amount, 10, amount))
            pygame.draw.line(screen, (0, 0, 0), (x, y-int(128*(target[k]/overall_total))), (x+10, y-int(128*(target[k]/overall_total))))
            y -= int(128*float(target[k])/overall_total)

        start, end, pressure = main.gm.machine.start_pressure, main.gm.machine.end_pressure, main.gm.machine.pressure
        pressure = min(100, pressure)
        if pressure >= start and pressure <= end:
            col = (0, 255, 0)
        else:
            col = (255, 0, 0)
        amount = int(128*(float(pressure)/100))
        screen.fill(col, pygame.Rect(28+156, 28+337+128-amount, 10, amount))
        amount = int(128*(float(start)/100))
        pygame.draw.line(screen, (0, 0, 0), (28+156, 28+337+128-amount), (28+156+12, 28+337+128-amount))
        amount = int(128*(float(end)/100))
        pygame.draw.line(screen, (0, 0, 0), (28+156, 28+337+128-amount), (28+156+12, 28+337+128-amount))

        # dial
        self.wheelrect.topleft = (114 - self.wheel.get_width()/2, 429 - self.wheel.get_height()/2)
        screen.blit(self.wheel, self.wheelrect)
        # end dial

        if main.state == main.PLAY:
            screen.blit(self.play, self.playrect)
        elif main.state == main.PAUSE:
            screen.blit(self.pause, self.playrect)

        screen.blit(clock, clockrect)
        screen.blit(score, scorerect)

    def click(self, pos, button):
        if self.quitrect.collidepoint(pos) and button == 1:
            main.gm.cursor.drop()
            main.state = main.MENU
            return True
        elif self.tilerect.collidepoint(pos) and button == 1:
            x, y = pos
            x, y = x-28, y-28
            if x >= xs[0] and x <= xs[0] + 32:
                col = 0
            elif x >= xs[1] and x <= xs[1] + 32:
                col = 1
            elif x >= xs[2] and x <= xs[2] + 32:
                col = 2
            else:
                return False
            if y >= ys[0] and y <= ys[0] + 32:
                row = 0
            elif y >= ys[1] and y <= ys[1] + 32:
                row = 1
            elif y >= ys[2] and y <= ys[2] + 32:
                row = 2
            else:
                return False
            if tiles[row][col] is not None:
                main.gm.cursor.pickUp(tiles[row][col]())
            return True
        elif self.tilerect.collidepoint(pos) and button == 3:
            main.gm.cursor.drop()
            return True
        elif self.wheelrect.collidepoint(pos) and button == 1:
            main.gm.boiler.increaseRate()
            self.wheel = pygame.transform.rotate(self.wheel_normal, -270*main.gm.boiler.rate)
            return True
        elif self.wheelrect.collidepoint(pos) and button == 3:
            main.gm.boiler.decreaseRate()
            self.wheel = pygame.transform.rotate(self.wheel_normal, -270*main.gm.boiler.rate)
            return True
        elif self.playrect.collidepoint(pos) and button == 1:
            main.gm.cursor.drop()
            if main.state == main.PLAY:
                main.state = main.PAUSE
            elif main.state == main.PAUSE:
                main.state = main.PLAY
            return True
        elif self.spannerrect.collidepoint(pos) and button == 1:
            main.gm.cursor.setType(main.gm.cursor.SPANNER)
            return True
        elif self.bombrect.collidepoint(pos) and button == 1:
            main.gm.cursor.setType(main.gm.cursor.BOMB)
            return True
        else:
            return False
