import xml.sax
import sys

from board import Board
from pieces import Boiler, Machine, Red, Green, Blue, Yellow
from infobox import InfoBox

class LevelSet:
    def __init__(self, filename):
         handler = LevelHandler()
         xml.sax.parse(filename, handler)
         self.levels = handler.levelset
         self.currentlevel = 0

    def setupGame(self, gm):
        level = self.levels[self.currentlevel]
        gm.board = Board()
        gm.boiler = Boiler()
        gm.machine = Machine()
        gm.machine.setTargets(level["targets"])
        gm.board.setPiece(level["boiler"][0], self.levels[self.currentlevel]["boiler"][1], gm.boiler)
        gm.board.setPiece(level["machine"][0], self.levels[self.currentlevel]["machine"][1], gm.machine)
        for k, obj in [("red", Red), ("green", Green), ("blue", Blue), ("yellow", Yellow)]:
            if level.has_key(k):
                gm.board.setPiece(level[k][0], self.levels[self.currentlevel][k][1], obj())

        gm.infobox = InfoBox(gm)
        gm.infobox.time_left = level["timelimit"]
        gm.score = 0

class LevelHandler(xml.sax.handler.ContentHandler):
    def __init__(self):
        xml.sax.handler.ContentHandler.__init__(self)
        self.levelset = None
        self.current = []

    def startElement(self, name, attrs):
        if name == "levels":
            self.levelset = []
        elif name == "level":
            self.current.append({})
        elif name == "timelimit":
            tl = 0
            if attrs.has_key("minutes"):
                tl += int(attrs["minutes"])*60
            if attrs.has_key("seconds"):
                tl += int(attrs["seconds"])
            if tl <= 0:
                print "Error, level set contains a level with an invalid timelimit"
                sys.exit(1)
            self.current[-1]["timelimit"] = tl
        elif name == "boiler":
            self.current[-1]["boiler"] = (int(attrs["x"]), int(attrs["y"]))
        elif name == "machine":
            self.current[-1]["machine"] = (int(attrs["x"]), int(attrs["y"]))
        elif name == "red":
            self.current[-1]["red"] = (int(attrs["x"]), int(attrs["y"]))
        elif name == "green":
            self.current[-1]["green"] = (int(attrs["x"]), int(attrs["y"]))
        elif name == "blue":
            self.current[-1]["blue"] = (int(attrs["x"]), int(attrs["y"]))
        elif name == "yellow":
            self.current[-1]["yellow"] = (int(attrs["x"]), int(attrs["y"]))
        elif name == "targets":
            self.current.append([])
        elif name == "target":
            d = {"min": int(attrs["min"]), "max": int(attrs["max"])}
            if attrs.has_key("O"):
                d["O"] = int(attrs["O"])
            if attrs.has_key("R"):
                d["R"] = int(attrs["R"])
            if attrs.has_key("G"):
                d["G"] = int(attrs["G"])
            if attrs.has_key("B"):
                d["B"] = int(attrs["B"])
            if attrs.has_key("Y"):
                d["Y"] = int(attrs["Y"])
            if attrs.has_key("rate"):
                d["rate"] = int(attrs["rate"])
            self.current[-1].append(d)
        else:
            raise SystemError, "Unknown tag in level file. %s" % (name, )

    def endElement(self, name):
        if name == "levels":
            pass
        elif name == "level":
            self.levelset.append(self.current.pop())
        elif name == "timelimit":
            pass
        elif name == "boiler":
            pass
        elif name == "machine":
            pass
        elif name == "red":
            pass
        elif name == "green":
            pass
        elif name == "blue":
            pass
        elif name == "yellow":
            pass
        elif name == "targets":
            targets = self.current.pop()
            self.current[-1]["targets"] = targets
        elif name == "target":
            pass