import pygame

import gfx
import game
import menu
import intro
import instructions

MENU = 1
PLAY = 2
BEGIN_GAME = 3
LEVEL_INTRO = 4
LEVEL_OUTRO = 5
GAME_WIN = 6
GAME_LOSE = 7
VIEW_HIGH_SCORES = 8
ENTER_HIGH_SCORE = 9
PAUSE = 10
INSTRUCTIONS = 11

gm = None
mu = None
state = MENU
outro = None
endscreen = None

def main():
    global gm, mu, state

    pygame.init()

    screen = pygame.display.set_mode((800, 600), pygame.DOUBLEBUF | pygame.HWSURFACE)
    pygame.display.set_caption("Steam Pipe Dreams")

    #world = pygame.sprite.Group()
    bg = gfx.Background()
    gm = game.Game()
    gm.levels.currentlevel = 0
    mu = menu.Menu()
    instruct = instructions.Instructions()

    quit = False
    clk, frames = pygame.time.Clock(), 0
    while not quit:
        clk.tick()
        frames += 1
        if frames == 1000:
            print "%.3f fps" % (clk.get_fps(), )
            frames = 0

        if state == BEGIN_GAME:
            gm.levels.setupGame(gm)
            inro = intro.Intro()
            state = LEVEL_INTRO

        # do event handling
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                quit = True
            elif event.type == pygame.MOUSEBUTTONUP:
                if state == MENU:
                    mu.click(event.pos, event.button)
                elif state == PLAY:
                    gm.click(event.pos, event.button)
                elif state == PAUSE:
                    gm.infobox.click(event.pos, event.button)
                elif state == LEVEL_INTRO:
                    inro.click(event.pos, event.button)
                elif state == LEVEL_OUTRO:
                    outro.click(event.pos, event.button)
                elif state in [GAME_WIN, GAME_LOSE]:
                    endscreen.click(event.pos, event.button)
                elif state == INSTRUCTIONS:
                    instruct.click(event.pos, event.button)
        # done event handling

        t = clk.get_time()/1000.0
        if state == MENU:
            mu.update(t)
        elif state == PLAY:
            gm.update(t)
        elif state == LEVEL_INTRO:
            inro.update(t)
        elif state == LEVEL_OUTRO:
            outro.update(t)
        elif state in [GAME_WIN, GAME_LOSE]:
            endscreen.update(t)

        screen.blit(bg.image, bg.rect)
        if state == MENU:
            gm.cursor.drop()
            mu.render(screen)
        elif state == PLAY:
            gm.render(screen)
        elif state == PAUSE:
            gm.infobox.render(screen)
            gm.cursor.render(screen)
            paused = gfx.loadImage("paused.png")
            pausedrect = paused.get_rect()
            pausedrect.topleft = (400, 280)
            screen.blit(paused, pausedrect)
        elif state == LEVEL_INTRO:
            inro.render(screen)
        elif state == LEVEL_OUTRO:
            outro.render(screen)
        elif state in [GAME_WIN, GAME_LOSE]:
            endscreen.render(screen)
        elif state == INSTRUCTIONS:
            instruct.render(screen)

        pygame.display.flip()
