import pygame
import math

from pipedreams.gfx import loadImage

class Piece(object):
    LEFT = 1
    RIGHT = 2
    UP = 3
    DOWN = 4

    def __init__(self):
        self.rect = pygame.Rect(0, 0, 32, 32)

        self.left, self.right, self.up, self.down = None, None, None, None
        self.waiting = []
        self.graph = False
        self.require_graph = False
        self.no_graph = False
        self.pipe = False
        self.leakage = 0.5
        self._steam = {"O": 0, "R": 0, "G": 0, "B": 0, "Y": 0}
        if not hasattr(Piece, "guageimg"):
            Piece.guageimg = loadImage("guage.png")

    def update(self, time):
        for s in self.waiting:
            for c in s.keys():
                self._steam[c] += s[c]
        self.releaseSteam(self.leakage)
        self.waiting = []

    def render(self, screen):
        screen.blit(self.image, self.rect)
        O, R, G, B, Y = float(self.O), float(self.R), float(self.G), float(self.B), float(self.Y)
        total = O + R + G + B + Y
        if self.graph and total > 0.0:
            screen.blit(self.guageimg, self.rect)
            O, R, G, B, Y = O / total, R / total, G / total, B / total, Y / total
            x = 10 + self.rect.topleft[0]
            y1 = 14 + self.rect.topleft[1]
            y2 = 7
            if O > 0.0:
                pygame.draw.rect(screen, (225, 225, 225), pygame.Rect(x, y1, int(O * 13), y2), 0)
                x += int(O * 13)
            if R > 0.0:
                pygame.draw.rect(screen, (255, 0, 0), pygame.Rect(x, y1, int(R * 13), y2), 0)
                x += int(R * 13)
            if G > 0.0:
                pygame.draw.rect(screen, (0, 255, 0), pygame.Rect(x, y1, int(G * 13), y2), 0)
                x += int(G * 13)
            if B > 0.0:
                pygame.draw.rect(screen, (0, 0, 255), pygame.Rect(x, y1, int(B * 13), y2), 0)
                x += int(B * 13)
            if Y > 0.0:
                pygame.draw.rect(screen, (255, 255, 0), pygame.Rect(x, y1, int(Y * 13), y2), 0)
            angle = -min(1.0, (self.pressure / 100)) * (math.pi/2.0) + (math.pi*5.0/4.0)
            pygame.draw.line(screen, (0, 0, 0), (15 + self.rect.topleft[0], 20 + self.rect.topleft[1]),
                                                (15 + self.rect.topleft[0] + math.sin(angle)*10, 20 + self.rect.topleft[1] + math.cos(angle)*10), 1)

    def acceptSteam(self, steam):
        self.waiting.append(steam)

    def transferSteam(self, other):
        if self.pressure > other.pressure:
            steam = float(self.steam)
            size = min(max(1, 5*(self.pressure - other.pressure)), steam/2.0)
            other.acceptSteam({"O": self.O*(size/steam),
                               "R": self.R*(size/steam),
                               "G": self.G*(size/steam),
                               "B": self.B*(size/steam),
                               "Y": self.Y*(size/steam)})
            self.releaseSteam(size)

    def releaseSteam(self, amount):
        if self.steam <= 0:
            return
        steam = amount/float(self.steam)
        self._steam = {"O": self.O*(1.0-steam),
                       "R": self.R*(1.0-steam),
                       "G": self.G*(1.0-steam),
                       "B": self.B*(1.0-steam),
                       "Y": self.Y*(1.0-steam)}

    def adjustUp(self):
        pass
    def adjustDown(self):
        pass

    def __get_pressure(self):
        return float(sum([x for x in self._steam.values()])) / self.volume
    pressure = property(__get_pressure, None)
    def __get_steam(self):
        return float(sum([x for x in self._steam.values()]))
    steam = property(__get_steam, None)

    def __get_O(self):
        return self._steam["O"]
    def __set_O(self, value):
        self._steam["O"] = value
    O = property(__get_O, __set_O)
    def __get_R(self):
        return self._steam["R"]
    def __set_R(self, value):
        self._steam["R"] = value
    R = property(__get_R, __set_R)
    def __get_G(self):
        return self._steam["G"]
    def __set_G(self, value):
        self._steam["G"] = value
    G = property(__get_G, __set_G)
    def __get_B(self):
        return self._steam["B"]
    def __set_B(self, value):
        self._steam["B"] = value
    B = property(__get_B, __set_B)
    def __get_Y(self):
        return self._steam["Y"]
    def __set_Y(self, value):
        self._steam["Y"] = value
    Y = property(__get_Y, __set_Y)
