import math
import pygame

from pipedreams.gfx import loadImage
from piece import Piece

class ReleaseHorizontal(Piece):
    def __init__(self):
        Piece.__init__(self)

        self.image = loadImage("release_horizontal.png")
        self.accept = [Piece.LEFT, Piece.RIGHT]
        self.pipe = True
        self.require_graph = True
        self.volume = 10
        self.rotate = ReleaseVertical
        self.limit = 100

    def update(self, time):
        Piece.update(self, time)
        if self.pressure > self.limit:
            ideal = self.limit * self.volume
            self.releaseSteam((self.steam - self.limit) / 5)
            from pipedreams.main import gm
            gm.board.smoke.addSteam(self.rect.topleft[0] + 16, self.rect.topleft[1] + 1)
            gm.board.smoke.addSteam(self.rect.topleft[0] + 16, self.rect.topleft[1] + 30)
        if self.pressure > 1 and self.left and Piece.RIGHT in self.left.accept:
            self.transferSteam(self.left)
        if self.pressure > 1 and self.right and Piece.LEFT in self.right.accept:
            self.transferSteam(self.right)

    def render(self, screen):
        Piece.render(self, screen)
        if self.graph and self.steam > 0.0:
            angle = -(self.limit / 100.0) * (math.pi/2.0) + (math.pi*5.0/4.0)
            pygame.draw.line(screen, (255, 0, 0), (15 + self.rect.topleft[0], 20 + self.rect.topleft[1]),
                                            (15 + self.rect.topleft[0] + math.sin(angle)*10, 20 + self.rect.topleft[1] + math.cos(angle)*10), 1)

    def adjustUp(self):
        self.limit = min(100, self.limit + 10)
    def adjustDown(self):
        self.limit = max(0, self.limit - 10)

class ReleaseVertical(Piece):
    def __init__(self):
        Piece.__init__(self)

        self.image = loadImage("release_vertical.png")
        self.accept = [Piece.UP, Piece.DOWN]
        self.pipe = True
        self.require_graph = True
        self.volume = 10
        self.rotate = ReleaseHorizontal
        self.limit = 100

    def update(self, time):
        Piece.update(self, time)
        if self.pressure > self.limit:
            ideal = self.limit * self.volume
            self.releaseSteam((self.steam - self.limit) / 5)
            from pipedreams.main import gm
            gm.board.smoke.addSteam(self.rect.topleft[0] + 1, self.rect.topleft[1] + 16)
            gm.board.smoke.addSteam(self.rect.topleft[0] + 30, self.rect.topleft[1] + 16)
        if self.pressure > 1 and self.down and Piece.UP in self.down.accept:
            self.transferSteam(self.down)
        if self.pressure > 1 and self.up and Piece.DOWN in self.up.accept:
            self.transferSteam(self.up)

    def render(self, screen):
        Piece.render(self, screen)
        if self.graph and self.steam > 0.0:
            angle = -(self.limit / 100.0) * (math.pi/2.0) + (math.pi*5.0/4.0)
            pygame.draw.line(screen, (255, 0, 0), (15 + self.rect.topleft[0], 20 + self.rect.topleft[1]),
                                            (15 + self.rect.topleft[0] + math.sin(angle)*10, 20 + self.rect.topleft[1] + math.cos(angle)*10), 1)

    def adjustUp(self):
        self.limit = min(100, self.limit + 10)
    def adjustDown(self):
        self.limit = max(0, self.limit - 10)
